package com.hisschemoller.utils.audio.processors 
{
	import com.hisschemoller.model.AudioDriverProxy;
	import com.hisschemoller.utils.audio.vo.PitchedSampleProgramVO;
	import com.hisschemoller.utils.audio.vo.ProgramVO;
	import com.hisschemoller.utils.audio.vo.SampleProgramVO;
	
	import flash.utils.ByteArray;	
	/**
	 * Plays part of an MP3 file (a slice) from _startSampleIndex to _endSampleIndex.
	 * 
	 * @author Wouter Hisschemöller
	 * (c) Apr 30, 2009
	 */
	public class SimpleSampleProcessor implements IAudioProcessor 
	{
		private const BYTE_ARRAY : ByteArray = new ByteArray();
		private var _programVO : SampleProgramVO;
		private var _deltaSamples : int;
		private var _pitch : int;
		private var _velocity : int;
		private var _sourceByteArray : ByteArray;
		private var _startSampleIndex : int;
		private var _endSampleIndex : int;
		private var _currentSampleIndex : int;
		private var _relativeSampleIndex : int;
		private var _attackDuration : int = 100;
		private var _decayDuration : int = 100;
		private var _gain : Number = 1;
		
		/**
		 * @param programVO : Instrument program data object.
		 * @param deltaSamples : offset into the buffer where to start writing.
		 */
		public function SimpleSampleProcessor(programVO : PitchedSampleProgramVO, deltaSamples : int = 0, pitch : int = 60, velocity : int = 100)
		{
			_programVO = programVO;
			_deltaSamples = deltaSamples;
			_pitch = pitch;
			_velocity = velocity;
			_sourceByteArray = programVO.soundData.byteArray;
			_startSampleIndex = programVO.start;
			_endSampleIndex = programVO.end;
			_currentSampleIndex = _startSampleIndex;
		}

		/**
		 * @param buffer : stereo buffer of AudioDriver.BUFFER_SIZE number of samples
		 * @param bufferStartIndex: From where in the buffer to start to write.
		 * @return Boolean : true if the playback is done and this generator can be removed
		 */
		public function process(buffer : Vector.<Vector.<Number>>, bufferStartIndex : uint, bufferLength : uint) : Boolean
		{
			/** if the end of the slice is reached within the current buffer, adjust the samples to read. */
			if(_currentSampleIndex + bufferLength <= _endSampleIndex)
			{
				var numSamples : int = bufferLength;
			}
			else
			{
				numSamples = _endSampleIndex - _currentSampleIndex;
			}
			
			BYTE_ARRAY.position = 0;
			BYTE_ARRAY.writeBytes(_sourceByteArray, _currentSampleIndex * AudioDriverProxy.BYTES_PER_SAMPLE, numSamples * AudioDriverProxy.BYTES_PER_SAMPLE);
			BYTE_ARRAY.position = 0;
			
			var l : Vector.<Number> = buffer[0];
			var r : Vector.<Number> = buffer[1];
			
			for(var i : int = _deltaSamples ;i < numSamples; ++i)
			{
				/** A short attack to remove clicks at the start of the slice */
				if(_relativeSampleIndex + i < _attackDuration)
				{
					_gain = (_relativeSampleIndex + i) / _attackDuration;
				}
				
				/** A short decay to remove clicks at the end of the slice. */
				if(_endSampleIndex - _relativeSampleIndex + i < _decayDuration)
				{ 
					_gain = (_endSampleIndex - _relativeSampleIndex + i) / _decayDuration;
				}
				
				l[i] += BYTE_ARRAY.readFloat() * _gain;
				r[i] += BYTE_ARRAY.readFloat() * _gain;
			}
			
			/** If slice is done, fill the rest of the buffer with silence. */
			if(numSamples < AudioDriverProxy.BUFFER_BLOCK_SIZE)
			{
				numSamples = AudioDriverProxy.BUFFER_BLOCK_SIZE;

				for(;i < numSamples; ++i)
				{
					l[i] += 0.0;
					l[i] += 0.0;
				}
			}
			
			_currentSampleIndex += AudioDriverProxy.BUFFER_BLOCK_SIZE - _deltaSamples;
			_relativeSampleIndex = _currentSampleIndex - _startSampleIndex;
			_deltaSamples = 0;
			
			if(_currentSampleIndex >= _endSampleIndex) return true;
			
			return false;
		}
		
		/**
		 * 
		 */
		public function stop() : void
		{
			/** If possible leave the short decay to have a nice smooth end to the sound. */
			_endSampleIndex = Math.min(_currentSampleIndex + _decayDuration, _endSampleIndex);
		}
		
		/**
		 * @param control: Controller nr to change.
		 * @param value: New value for the control (between 0 and 1).
		 */
		public function controlChange(control : uint, value : Number) : void
		{
			
		}
		
		/**
		 * 
		 */
		public function get pitch() : int
		{
			return _pitch;
		}
		
		/**
		 * 
		 */
		public function get program() : ProgramVO
		{
			return _programVO;
		}
		
		/**
		 * 
		 */
		public function get sampleIndex() : int
		{
			return _currentSampleIndex;
		}
	}
}
